@@ -1371,6 +1371,11 @@ class StandardFunction:
 
             index = part_idx[part_id]
             data = part_data[part_id]
+            if len(index) == 0:
+                part_max_elment[part_id] = np.empty(int(nr_largest_elements))
+                part_max_elment[part_id].fill(np.NaN)
+                continue
+
             element_ids = np.unique(index[:, 0])
 
             for i, element_id in enumerate(element_ids):
