@@ -95,10 +95,40 @@ class DefintionObject(object):
         """
         self._elements[eid] = eid
 
+    def add_elements(self, dummy_part_id, eids):
+        """
+
+        Args:
+          eids:
+
+        Returns:
+
+        """
+
+        self._parts[dummy_part_id] = eids
+
+        # self._elements[eid] = eid
+
     def get_parts(self):
         """ """
         return sorted(self._parts)
 
+    def is_part_element_set(self, pid):
+        """
+        """
+        if pid in self._parts:
+            return isinstance(self._parts[pid], list)
+
+        return False
+
+    def get_part_element_set(self, pid):
+        """
+        """
+        if pid in self._parts:
+            return self._parts[pid]
+
+        return []
+
     def get_element(self):
         """ """
         return sorted(self._elements)
@@ -380,6 +410,20 @@ class DynasaurDefinitions:
         """
         return self._objects[tmp_object].get_parts()
 
+    def get_element_sets_of_defined_object(self, tmp_object):
+        """
+
+        Args:
+          tmp_object: type  str:
+
+        Returns:
+          array:: the parts as defined in ids.def:
+
+        """
+        for i in self._objects[tmp_object].get_parts():
+            print(i)
+        return self._objects[tmp_object].get_part_element_set()
+
     def get_info(self):
         """get info"""
         return self._info
@@ -442,6 +486,25 @@ class DynasaurDefinitions:
 
         return tmp_object in self._objects
 
+    def is_object_element_set(self, tmp_object):
+        """
+        checks if the tmp_object is defined as element sets
+
+        """
+        parts = [self._objects[tmp_object].is_part_element_set(part_id) for part_id in self._objects[tmp_object].get_parts()]
+
+        return all(parts)  # isinstance(a, list)
+
+    def get_object_element_set(self, tmp_object):
+        """
+        checks if the tmp_object is defined as element sets
+
+        """
+        element_sets = [self._objects[tmp_object].get_part_element_set(part_id) for part_id in self._objects[tmp_object].get_parts()]
+
+        return element_sets # isinstance(a, list)
+
+
     def __set_object(self, json_object):
         """
         set object
@@ -460,9 +523,10 @@ class DynasaurDefinitions:
                 self._objects[name].add_part(i)
 
         # TODO: Check if we need elem?
-        elif "elem" in json_object:
-            for elem in json_object["elem"]:
-                self._objects[name].add_element(elem)
+        elif "elem_set" in json_object:
+            for i, elems in enumerate(json_object["elem_set"]):
+                if isinstance(elems, list):
+                    self._objects[name].add_elements(i, elems)
 
         elif "elem_range" in json_object:
             for i in range(json_object["elem_range"][0], json_object["elem_range"][1]):
