@@ -279,73 +279,106 @@ class EloutObject(BinoutData):
 
         part_ids = []
 
-        if not self._dynasaur_definitions.is_object_defined(tmp_object=ID):
-            self._logger.emit(LOGConstants.WARNING[0],
-                              "OBJECT " + ID + " is not defined in your object definition file")
-        else:
-            part_ids = self.get_part_ids_by_object_name(ID, strain_stress_idx)
-            part_ids_defined = self._dynasaur_definitions.get_parts_of_defined_object(ID)
-            if len(part_ids) != len(part_ids_defined):
-                unresolved_part_ids = [part_id_defined for part_id_defined in part_ids_defined if part_id_defined not in part_ids]
-
-                self._logger.emit(LOGConstants.WARNING[0],
-                                  "Unresolved parts in your object definition with ID " + ID + " : " + ", ".join([str(i) for i in unresolved_part_ids])
-                                  + ". Available parts are : " + ", ".join([str(i)for i in self._get_part_ids(strain_stress_idx)]))
-
         element_count = 0
 
+        part_data = {}
+        part_index = {}
+        element_type = None
+        part_volume = None
+        el_by_part_id_el_id = None
+
         for (dynasaur_data_type, offset, delta_t) in data_offsets:
             if dynasaur_data_type == type:
                 data_offset = offset
                 data_delta_t = delta_t
 
+        def log_to_green_lagrange_strain(log_strain):
+            """
+                Takes the principle logarithmic strains and returns the principle Green-Lagrange strains
+
+                Args:
+                    log_strain: principle strains (time_steps x object_elems x 3)
+
+                Returns: Converted Green Lagrange strain
+            """
+
+            return (np.exp(log_strain) ** 2 - 1) / 2
+
         time_step_indices = list(range(data_offset, data_delta_t))
 
-        part_data = {}
-        part_index = {}
+        if self._dynasaur_definitions.is_object_defined(tmp_object=ID):
 
-        # if len(part_ids) == 0:
-        #     {"part_ids": part_ids, "part_data": part_data,
-        #      "part_idx": part_index, "time_step_indices": time_step_indices,
-        #      "element_type": element_type, "part_value": part_volume,
-        #      "el_by_part_id_el_id": el_by_part_id_el_id,
-        #      "time": self.get_interpolated_time()[time_step_indices]}
+            # check if the defined object consists of element_sets
+            if self._dynasaur_definitions.is_object_element_set(tmp_object=ID):
+                element_sets = self._dynasaur_definitions.get_object_element_set(tmp_object=ID)
 
+                for element_set_id, element_set in enumerate(element_sets):
+                    part_index[element_set_id] = []
+                    part_data[element_set_id] = []
+                    for element_id in element_set:
+                        part_id = self.get_part_id_by_element_ids(element_id, strain_stress_idx)
+                        if part_id is None:
+                            continue
 
-        #assert (len(part_ids) > 0)
+                        index, data = self.get_part_data(strain_stress_idx, part_id, element_id=element_id,
+                                                         time_step_indices=time_step_indices)
 
-        element_type = None
-        part_volume = None
-        el_by_part_id_el_id = None
+                        if strain_stress_type == "Green_Lagrange":
+                            data = log_to_green_lagrange_strain(data)
 
-        for part_id in part_ids:
-            index, data = self.get_part_data(strain_stress_idx, part_id, time_step_indices=time_step_indices)
+                        if len(part_data[element_set_id]) == 0:
+                            part_data[element_set_id] = data
+                        else:
+                            part_data[element_set_id] = np.hstack((part_data[element_set_id], data))
 
-            def log_to_green_lagrange_strain(log_strain):
-                """
-                    Takes the principle logarithmic strains and returns the principle Green-Lagrange strains
+                        if len(part_index[element_set_id]) == 0:
+                            part_index[element_set_id] = index
+                        else:
+                            part_index[element_set_id] = np.vstack((part_index[element_set_id], index))
 
-                    Args:
-                        log_strain: principle strains (time_steps x object_elems x 3)
+                        element_ids = self.get_element_ids_of_part_data(index)
+                        # print(", ".join([str(i) for i in element_ids]))
+                        element_count += len(element_ids)
+                        element_type = self.get_element_type_from_part_id(part_id, strain_stress_idx)
 
-                    Returns: Converted Green Lagrange strain
-                """
+                        part_volume = self.get_part_volume_by_part_ID()
+                        el_by_part_id_el_id = self.get_element_volume_by_part_id_and_element_id()
 
-                return (np.exp(log_strain) ** 2 - 1) / 2
+                part_ids = part_data.keys()
 
-            if strain_stress_type == "Green_Lagrange":
-                data = log_to_green_lagrange_strain(data)
+            # object defined through parts
+            else:
+                part_ids = self.get_part_ids_by_object_name(ID, strain_stress_idx)
+                part_ids_defined = self._dynasaur_definitions.get_parts_of_defined_object(ID)
+                if len(part_ids) != len(part_ids_defined):
+                    unresolved_part_ids = [part_id_defined for part_id_defined in
+                                           part_ids_defined if part_id_defined not in part_ids]
 
-            part_data[part_id] = data
-            part_index[part_id] = index
+                    self._logger.emit(LOGConstants.WARNING[0],
+                                      "Unresolved parts in your object definition with ID " + ID + " : " + ", ".join([str(i) for i in unresolved_part_ids])
+                                      + ". Available parts are : " + ", ".join([str(i)for i in self._get_part_ids(strain_stress_idx)]))
 
-            element_ids = self.get_element_ids_of_part_data(index)
-            element_count += len(element_ids)
-            element_type = self.get_element_type_from_part_id(part_id, strain_stress_idx)
+                for part_id in part_ids:
+                    index, data = self.get_part_data(strain_stress_idx, part_id, time_step_indices=time_step_indices)
 
-            # TODO: check if volume is defined
-            part_volume = self.get_part_volume_by_part_ID()
-            el_by_part_id_el_id = self.get_element_volume_by_part_id_and_element_id()
+                    if strain_stress_type == "Green_Lagrange":
+                        data = log_to_green_lagrange_strain(data)
+
+                    part_data[part_id] = data
+                    part_index[part_id] = index
+
+                    element_ids = self.get_element_ids_of_part_data(index)
+                    # print(", ".join([str(i) for i in element_ids]))
+                    element_count += len(element_ids)
+                    element_type = self.get_element_type_from_part_id(part_id, strain_stress_idx)
+
+                    # TODO: check if volume is defined
+                    part_volume = self.get_part_volume_by_part_ID()
+                    el_by_part_id_el_id = self.get_element_volume_by_part_id_and_element_id()
+
+        else:
+            self._logger.emit(LOGConstants.WARNING[0],
+                              "OBJECT " + ID + " is not defined in your object definition file")
 
         return {"part_ids": part_ids,
                 "part_data": part_data,
@@ -563,6 +596,9 @@ class EloutObject(BinoutData):
         Returns:
 
         """
+        if part_id not in self._data_mapping_part_id_to_element_type[strain_stress_type]:
+            return None
+
         element_type = self._data_mapping_part_id_to_element_type[strain_stress_type][part_id]
         return self._data[strain_stress_type][element_type]
 
@@ -600,7 +636,8 @@ class EloutObject(BinoutData):
 
         # merge lower and upper integration points for
         if name.startswith("eps_") and elout_elem == "shell":
-            # flatted_data_array = np.list(sum(self._binout.read("elout", elout_elem, name),())) list(sum(self._binout.read("elout", elout_elem, name),()))
+            # flatted_data_array = np.list(sum(self._binout.read("elout", elout_elem, name),()))
+            # list(sum(self._binout.read("elout", elout_elem, name),()))
             lower = list(sum(self._binout.read("elout", elout_elem, "lower_" + name), ()))
             upper = list(sum(self._binout.read("elout", elout_elem, "upper_" + name), ()))
             nr_elements = len(lower)
@@ -631,7 +668,7 @@ class EloutObject(BinoutData):
         mat = []
         ids = []
         ipts = []
-        if ((elout_elem == "shell" or elout_elem == "beam") and what == "stress"):
+        if (elout_elem == "shell" or elout_elem == "beam") and what == "stress":
             # ok
             nip = np.tile(self._binout.read(self._name, elout_elem, 'nip')[0], time.shape[0]).flatten()
             if len(nip) != 0:
@@ -641,7 +678,7 @@ class EloutObject(BinoutData):
                 ids = np.repeat(np.tile(self._binout.read(self._name, elout_elem, 'ids')[0], time.shape[0]).flatten(),
                                 nip).reshape((-1, 1))
 
-        elif (elout_elem == "shell" and what == "strain"):
+        elif elout_elem == "shell" and what == "strain":
             # ok
             nr_elements = time.shape[0] if isinstance(self._binout.read(self._name, elout_elem, 'ids')[0],
                                                       np.intp) else len(
@@ -747,8 +784,6 @@ class EloutObject(BinoutData):
             self._logger.emit(LOGConstants.WARNING[0], "Evaluated object contains failed elements!")
             w_stress_strain[ind] = np.tile(np.array([np.NaN]), 3)
 
-    ####################################################################################################################
-
     def _assign_stresses(self, elout_elem):
         """calculate stress from the elout_elem
 
@@ -1066,6 +1101,24 @@ class EloutObject(BinoutData):
         """
         return self._data_mapping_part_id_to_element_ids[strain_stress_type][part_id]
 
+    def get_part_id_by_element_ids(self, element_id, strain_stress_type):
+        """
+
+        Args:
+          element_id: param strain_stress_type:
+          strain_stress_type:
+
+        Returns:
+
+        """
+        for element_type in self._data_mapping_element_id_to_part_id[strain_stress_type]:
+            if element_id in self._data_mapping_element_id_to_part_id[strain_stress_type][element_type]:
+                return self._data_mapping_element_id_to_part_id[strain_stress_type][element_type][element_id]
+
+        else:
+            self._logger.emit(LOGConstants.WARNING[0], f'Element ID {element_id} not found')
+            return
+
     def get_part_data(self, strain_stress_type, part_id, element_id=None, time_step_indices=None):
         """description: return filtered index matrix and data tensor over all time steps for a given part_id
             will return all element, part, integration combinations
