@@ -11,10 +11,11 @@ class VPSData:
 
     def __init__(self, vps_file_path):
 
-        self.file_data = {}             # should be wrapped to binout data.
+        self.file_data = {}  # should be wrapped to binout data.
 
         self._vps_file_path = vps_file_path
         self._d = dict()
+        self._ids = dict()
 
         with h5py.File(self._vps_file_path, 'r') as f:
             f['CSMEXPL/multistate'].visititems(self._get_available_data_types_dict)
@@ -34,6 +35,8 @@ class VPSData:
         """
         if isinstance(node, h5py.Dataset):
             if node.name.split("/")[-1] == 'res':  # result of data
+                print(node.name)
+
                 if node.parent.name.split("/")[5] in [VPSDataConstant.NODE]:
                     self._set_node_elements(node)
 
@@ -64,34 +67,68 @@ class VPSData:
         if args[1] == "time":
             with h5py.File(self._vps_file_path, 'r') as f:
                 path = "/CSMEXPL/multistate/TIMESERIES1/multientityresults/MODEL/TIME/ZONE1_set1/erfblock/res"
-                return(f[path][()][:,0,0])
+                return (f[path][()][:, 0, 0])
 
         elif args[1] == "ids":
+
+            # check if data has been cached
+            if args[0] in self._ids:
+                return self._ids[args[0]]
+
             with h5py.File(self._vps_file_path, 'r') as f:
                 if args[0] == VPSDataConstant.SECFORC:
-                    entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.SECTION + "/Section_Force/ZONE1_set1/erfblock/entid"][()]
+                    # entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.SECTION + "/Section_Force/ZONE1_set1/erfblock/entid"][()]
+                    entids = f["/CSMEXPL/constant/attributes/SECTION/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/SECTION/erfblock/entid"][()]
                     uids = f["/CSMEXPL/constant/identifiers/" + VPSDataConstant.SECTION + "/erfblock/uid"][()]
 
                 elif args[0] == VPSDataConstant.RCFORC:
-                    entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.CONTACT+ "/Contact_Force/ZONE1_set1/erfblock/entid"][()]
-                    uids = f["/CSMEXPL/constant/identifiers/" + VPSDataConstant.CONTACT + "/erfblock/uid"][()]
+                    # entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.CONTACT + "/Contact_Force/ZONE1_set1/erfblock/entid"][()]
+                    entids = f["/CSMEXPL/constant/attributes/CONTACT/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/CONTACT/erfblock/entid"][()]
+                    uids = f["/CSMEXPL/constant/identifiers/CONTACT/erfblock/uid"][()]
 
                 elif args[0] == VPSDataConstant.NODOUT:
-                    entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.NODE + "/Acceleration/ZONE1_set1/erfblock/entid"][()]
-                    uids = f["/CSMEXPL/constant/identifiers/" + VPSDataConstant.NODE + "/erfblock/uid"][()]
+                    entids = f["/CSMEXPL/constant/attributes/NODE/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/NODE/erfblock/entid"][()]
+                    uids = f["/CSMEXPL/constant/identifiers/NODE/erfblock/uid"][()]
+
+                    # might be a help for debugging
+
+                    # entids = np.genfromtxt(r"01_const_attributes_entid.txt", dtype=np.int)
+                    # entid_translation = np.genfromtxt(r"03_const_identifiers_node_entid.txt", dtype=np.int)
+                    # uids = np.genfromtxt(r"04_const_identifiers_node_uid.txt", dtype=np.int)
+                    #
+                    # import csv
+                    # with open(r"02_const_attributes_title.txt", newline='') as f:
+                    #     reader = csv.reader(f)
+                    #     title = list(reader)
 
                 elif args[0] == VPSDataConstant.MATSUM:
-                    entids = f["/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + VPSDataConstant.ENERGY_PART + "/MINT/ZONE1_set1/erfblock/entid"][()]
+                    entids = f["/CSMEXPL/constant/attributes/PART/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/PART/erfblock/entid"][()]
                     uids = f["/CSMEXPL/constant/identifiers/" + VPSDataConstant.ENERGY_PART + "/erfblock/uid"][()]
 
                 elif args[0] == VPSDataConstant.GLSTAT:
-
                     return [0]
 
                 else:
                     assert False
 
-                return uids[entids-1][:, 0]
+                assert len(entid_translation) == len(uids[:, 0])
+
+                # TODO
+                #  Guess that could be done more efficiently
+                # entids is a list with length of the defined outputs (from the user)
+                #    entid_translation maps the entids with the uids. For each entid
+                #    we have to find the indices correct index in the entid_translation.
+                #    With the index we are able to extract the correct ids of the channels written out
+                #
+                output_uids_idx = [list(entid_translation).index(i) for i in entids if i in list(entid_translation)]
+
+                ids = uids[output_uids_idx][:, 0]
+                self._ids[args[0]] = ids
+                return ids
 
         axis = 2
         if args[0] == VPSDataConstant.NODOUT:
@@ -204,7 +241,7 @@ class VPSData:
             data_channel = VPSDataConstant.ROTATION_ACCELERATION
 
         else:
-            #TODO: add console logger
+            # TODO: add console logger
             print("ID FAIL")
             return
 
@@ -233,7 +270,7 @@ class VPSData:
             idx = [0]
             data_channel = "DMSC"
         else:
-            #TODO: add console logger
+            # TODO: add console logger
             print("ID FAIL")
             return
 
@@ -300,7 +337,7 @@ class VPSData:
             idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
             data_channel = VPSDataConstant.SECTION_FORCE
         elif arg == DataChannelTypesSecforc.TOTAL_FORCE:
-            idx = [0,1,2]
+            idx = [0, 1, 2]
             data_channel = VPSDataConstant.SECTION_FORCE
 
         elif arg == DataChannelTypesSecforc.X_MOMENT or \
@@ -308,11 +345,11 @@ class VPSData:
             idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
             data_channel = VPSDataConstant.SECTION_MOMENT
         elif arg == DataChannelTypesSecforc.TOTAL_MOMENT:
-            idx = [0,1,2]
+            idx = [0, 1, 2]
             data_channel = VPSDataConstant.SECTION_MOMENT
 
         else:
-            #TODO: add console logger
+            # TODO: add console logger
             print("ID FAIL")
             return
 
@@ -327,7 +364,7 @@ class VPSData:
         Returns:
 
         """
-        if arg == None:
+        if arg is None:
             idx = 0
             data_channel = "abc"
         else:
@@ -347,7 +384,7 @@ class VPSData:
 
         """
         if arg == DataChannelTypesContact.X_FORCE or \
-             arg == DataChannelTypesContact.Y_FORCE or arg == DataChannelTypesSecforc.Z_FORCE:
+                arg == DataChannelTypesContact.Y_FORCE or arg == DataChannelTypesSecforc.Z_FORCE:
             idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
             data_channel = VPSDataConstant.CONTACT_FORCE
 
@@ -355,7 +392,7 @@ class VPSData:
             idx = [0, 1, 2]
             data_channel = VPSDataConstant.CONTACT_FORCE
         else:
-            #TODO: add console logger
+            # TODO: add console logger
             print("ID FAIL")
             return
 
@@ -418,8 +455,8 @@ class VPSData:
 
         if node.parent.name.split("/")[idx_of_data] == VPSDataConstant.CONTACT_FORCE:
             self._d[VPSDataConstant.RCFORC].extend([DataChannelTypesContact.X_FORCE,
-                                                     DataChannelTypesContact.Y_FORCE,
-                                                     DataChannelTypesContact.Z_FORCE])
+                                                    DataChannelTypesContact.Y_FORCE,
+                                                    DataChannelTypesContact.Z_FORCE])
 
     def _set_global_energy_elements(self, node):
         """
@@ -480,12 +517,7 @@ class VPSData:
         elif node.parent.name.split("/")[idx_of_data] == "MKIN":
             self._d[VPSDataConstant.MATSUM].extend(["kinetic_energy"])
 
-
-
-
-        #idx_of_data = node.parent.name.split("/").index(node.parent.name.split("/")[5]) + 1
-
-
+        # idx_of_data = node.parent.name.split("/").index(node.parent.name.split("/")[5]) + 1
 
     def _set_section_elements(self, node):
         """
@@ -519,5 +551,3 @@ class VPSData:
                                                          DataChannelTypesSecforc.Y_MOMENT,
                                                          DataChannelTypesSecforc.Z_MOMENT,
                                                          DataChannelTypesSecforc.TOTAL_MOMENT])
-
-
