@@ -210,20 +210,18 @@ class DataVisualizationController(PluginInterface, Plugin):
         if part_of not in self._data_dict.keys():
             self._data_dict.update({part_of: {}})
 
-        if visualization not in self._data_dict[part_of].keys():
-            y_unit = y_data.units if isinstance(y_data, pint.Quantity) else "dimensionless"
-            x_unit = x_data.units if isinstance(x_data, pint.Quantity) else "dimensionless"
-            x_data = x_data.magnitude if isinstance(x_data, pint.Quantity) else x_data
-            y_data = y_data.magnitude if isinstance(y_data, pint.Quantity) else y_data
-            self._data_dict[part_of].update({visualization: {"X": x_data,
-                                                             "x_label": x_label,
-                                                             "x_unit": x_unit,
-                                                             "Y": y_data,
-                                                             "y_label": y_label,
-                                                             "y_unit": y_unit}})
-        else:
-            self._data_dict[part_of][visualization].update({"X": x_data, "x_label": x_label,
-                                                            "Y": y_data, "y_label": y_label})
+        y_unit = y_data.units if isinstance(y_data, pint.Quantity) else "dimensionless"
+        x_unit = x_data.units if isinstance(x_data, pint.Quantity) else "dimensionless"
+        x_data = x_data.magnitude if isinstance(x_data, pint.Quantity) else x_data
+        y_data = y_data.magnitude if isinstance(y_data, pint.Quantity) else y_data
+
+        self._data_dict[part_of][visualization] = {"X": x_data,
+                                                         "x_label": x_label,
+                                                         "x_unit": x_unit,
+                                                         "Y": y_data,
+                                                         "y_label": y_label,
+                                                         "y_unit": y_unit}
+
 
     def _get_padded_data_dict(self):
         """
